/******************************************
Copyright 2014 Brent Ozar PLF, LLC (DBA Brent Ozar Unlimited)
See http://BrentOzar.com/go/eula for the End User Licensing Agreement.

Last update: 2014/11/21 - Include DBCC IND 
*******************************************/

/* This script is intended for test purposes only. */

/* 
Corrupt your database - on purpose! 
*/

/* Create database */
USE [master]; 
GO
IF DB_ID('CorruptMe') IS NOT NULL
BEGIN
	ALTER DATABASE CorruptMe SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE CorruptMe;
END

RESTORE DATABASE CorruptMe
	FROM DISK=N'D:\Software Downloads\Adventure Works 2014 Full Database Backup\AdventureWorks2014.bak'
	WITH MOVE 'AdventureWorks2014_Data' to 'C:\Program Files\Microsoft SQL Server\MSSQL12.MSSQLSERVER\MSSQL\DATA\CorruptMe_Data.mdf',
	MOVE 'AdventureWorks2014_Log' to 'C:\Program Files\Microsoft SQL Server\MSSQL12.MSSQLSERVER\MSSQL\DATA\CorruptMe_log.ldf';

ALTER DATABASE CorruptMe SET RECOVERY FULL;
GO 

/* Back up database */
BACKUP DATABASE CorruptMe to DISK=N'D:\SQL Backups\CorruptMe-Backup1.bak' 
	WITH COMPRESSION;
GO

/* Pick a table with a Clustered index and a Nonclustered index */ 
USE CorruptMe; 
GO 
SELECT *
FROM Purchasing.ShipMethod;
GO 

/* Back up log */
BACKUP LOG CorruptMe TO DISK=N'D:\SQL Backups\CorruptMe-LogBackup1.trn' 
	WITH COMPRESSION;
GO

/* Update table */ 
UPDATE Purchasing.ShipMethod 
SET Name = 'JES - TRUCK GROUND' 
WHERE ShipMethodID = 1; 

CHECKPOINT
GO 

/* Demo 1 - clustered index */ 
/* Find page number 
The sys.dm_db_database_page_allocations method will only work in SQL server 2012 + */
--Purchasing.ShipMethod IndexID=1 (clustered index)
SELECT allocated_page_page_id 
FROM sys.dm_db_database_page_allocations(DB_ID('CorruptMe'), OBJECT_ID('Purchasing.ShipMethod'), 1, NULL, 'detailed')
WHERE page_type_desc='DATA_PAGE'
	AND is_allocated = 1
ORDER BY allocated_page_page_id ASC;
GO
/* Page = 820 */

/* Prior to SQL Server 2012, use DBCC IND */ 
DBCC IND ('CorruptMe', 'Purchasing.ShipMethod', 1); 
/* Choose a page number, based on PageType of 1 or 2 */ 

/* Take the database offline */ 
USE master;
GO
ALTER DATABASE CorruptMe SET OFFLINE;
GO

/* 
		Navigate to D:\Software Downloads\xvi32
		Start xvi32 with elevated permissions (Right click -> Run as Administrator)

		Inside xvi32, open the file 'C:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\CorruptMe_Data.mdf'

		Address > Goto 
		Use decimal 

		Offset = pageid * 8192
		For page 820 = 820 * 8192 = 6717440
		Change data 
		Save and close the file 
*/ 

/* Bring database online */ 
ALTER DATABASE CorruptMe SET ONLINE;
GO

/* Try to view data */ 
USE CorruptMe;
GO

SELECT * 
FROM Person.Person; 
GO 

SELECT * 
FROM Purchasing.ShipMethod;
GO 

SELECT Name 
FROM Purchasing.ShipMethod;
GO 

/* Review SQL Server error log */ 

/* Detecting corruption */  
DBCC CHECKDB ('CorruptMe');
GO

DBCC CHECKDB ('CorruptMe') WITH NO_INFOMSGS;
GO 

/* Fixing corruption */ 
/* What object is that on? */ 
SELECT OBJ.object_id, OBJ.name, OBJ.type_desc, IND.index_id, IND.name 
FROM sys.objects OBJ 
	INNER JOIN sys.indexes IND ON IND.object_id = OBJ.object_id 
WHERE OBJ.object_id=78623323; 

/* Clustered index  - Restore the page */ 
/* This example uses an OFFLINE restore, which is applicable to all versions of SQL Server. 
Enterprise Edition allows you to do an ONLINE restore, which is a demo for a different day. */ 
USE master; 
GO 

/* This makes the database "offline" - you don't actually set it OFFLINE */ 
/* No, this isn't confusing at all, I know. */ 
BACKUP LOG CorruptMe TO Disk=N'D:\SQL Backups\CorruptMe-LogBackup2.trn'
	WITH NORECOVERY;
GO 

/* Restore full backup with PAGE */ 
RESTORE DATABASE CorruptMe 
	PAGE='1:820' --Have multiple? Separate with commmas. 
	FROM DISK=N'D:\SQL Backups\CorruptMe-Backup1.bak' 
	WITH NORECOVERY; 

/* Take page number out */
--? 
--RESTORE DATABASE CorruptMe  
--	FROM DISK=N'D:\SQL Backups\CorruptMe-Backup1.bak' 
--	WITH NORECOVERY; 

/* Restore log backups */ 
RESTORE LOG CorruptMe FROM Disk=N'D:\SQL Backups\CorruptMe-LogBackup1.trn' 
	WITH NORECOVERY; 

RESTORE LOG CorruptMe FROM Disk=N'D:\SQL Backups\CorruptMe-LogBackup2.trn' 
	WITH NORECOVERY;

/* IF you had Enterprise Edition, the "online" restore would have the log backup with norecovery here */
--/* Take another log backup */ 
--BACKUP LOG CorruptMe TO Disk=N'D:\SQL Backups\CorruptMe-LogBackup3.trn'
--	WITH NORECOVERY;
--GO 

--/* Restore the last log backup */ 
--RESTORE LOG CorruptMe FROM Disk=N'D:\SQL Backups\CorruptMe-LogBackup3.trn' 
--	WITH NORECOVERY;

RESTORE DATABASE CorruptMe 
	WITH RECOVERY; 



/* Is it fixed? */
USE CorruptMe; 
GO
SELECT * 
FROM Purchasing.ShipMethod;
GO 

DBCC CHECKDB ('CorruptMe') WITH NO_INFOMSGS;
GO 

/* End clustered index */ 

/* What if you didn't have a recent full and/or log backup? */ 



/* Demo 2 - Non-clustered index */ 
/* Find page number */
--Purchasing.ShipMethod IndexID=2 (nonclustered index) AK_ShipMethod_Name
SELECT allocated_page_page_id
FROM sys.dm_db_database_page_allocations(DB_ID('CorruptMe'), OBJECT_ID('Purchasing.ShipMethod'), 2, NULL, 'detailed')
WHERE page_type_desc='INDEX_PAGE' 
	AND is_allocated = 1
ORDER BY allocated_page_page_id ASC;
GO 
/* Page = 5258 */ 

/* Take the database offline */ 
USE master;
GO
ALTER DATABASE CorruptMe SET OFFLINE;
GO

/* 
		Navigate to D:\Software Downloads\xvi32
		Start xvi32 with elevated permissions (Right click -> Run as Administrator)

		Inside xvi32, open the file 'C:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\CorruptMe_Data.mdf'

		Address > Goto 
		Use decimal 

		Offset = pageid * 8192
			For page 5258 = 5258 * 8192 = 43073536
		Change data 
		Save and close the file 
*/ 

/* Bring database online */ 
ALTER DATABASE CorruptMe SET ONLINE;
GO

/* Try to view data */ 
USE CorruptMe;
GO

SELECT * 
FROM Person.Person; 
GO 

SELECT * 
FROM Purchasing.ShipMethod;
GO

SELECT Name 
FROM Purchasing.ShipMethod;
GO 

/* Review SQL Server error log */ 

/* Detecting corruption */  
DBCC CHECKDB ('CorruptMe') WITH NO_INFOMSGS;
GO 

/* Fixing corruption */ 
/* What object is that on? */ 
SELECT OBJ.object_id, OBJ.name, OBJ.type_desc, IND.index_id, IND.name 
FROM sys.objects OBJ 
INNER JOIN sys.indexes IND ON IND.object_id = OBJ.object_id 
WHERE OBJ.object_id=78623323; 

/* Non-clustered index */ 
ALTER INDEX AK_ShipMethod_Name ON Purchasing.ShipMethod 
DISABLE; 
ALTER INDEX AK_ShipMethod_Name ON Purchasing.ShipMethod 
REBUILD; 
GO 

/* Try to view data */ 
SELECT * 
FROM Purchasing.ShipMethod;
GO

SELECT Name 
FROM Purchasing.ShipMethod;
GO 

DBCC CHECKDB ('CorruptMe') WITH NO_INFOMSGS;
GO 
/* End nonclustered index */ 




/* Clean up */
USE master;
GO 
DROP DATABASE CorruptMe;
GO

--Delete backup files! 